SystemOrganization addCategory: #'SI-Wrapper'!
SystemOrganization addCategory: #'SI-Wrapper-Tests'!

TestCase subclass: #SignalWrapperTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SI-Wrapper-Tests'!

!SignalWrapperTest methodsFor: 'running' stamp: 'mt 6/14/2010 21:43:29.742'!
setUp

	super setUp.
	Smalltalk garbageCollect.
	Processor yield.! !

!SignalWrapperTest methodsFor: 'running' stamp: 'mt 6/14/2010 21:42:54.346'!
tearDown

	super tearDown.
	Smalltalk garbageCollect.
	Processor yield.! !

!SignalWrapperTest methodsFor: 'tests' stamp: 'mt 6/14/2010 22:36:51.161'!
test01ConvertToSignal

	| producer consumer |
	producer := SignalProducerByWrapping new.
	consumer := SignalConsumer new.

	self
		assert: 0
		equals: SignalProducerByWrapping wrapperCount.
		
	consumer connect: producer signal: #value: toSelector: #value:.
	
	self
		assert: 1
		equals: SignalProducerByWrapping wrapperCount.
		
	consumer connect: producer signal: #value: toSelector: #value:.
	
	self
		assert: 1
		equals: SignalProducerByWrapping wrapperCount.! !

!SignalWrapperTest methodsFor: 'tests' stamp: 'mt 6/14/2010 21:56:08.093'!
test02ConnectToConvertedSignal

	| producer consumer |
	producer := SignalProducerByWrapping new.
	consumer := SignalConsumer new.
		
	consumer connect: producer signal: #value: toSelector: #value:.
	producer value: #foobar.
	
	self
		assert: #foobar
		equals: consumer value.! !

!SignalWrapperTest methodsFor: 'tests' stamp: 'mt 6/14/2010 22:08:02.979'!
test03ConnectConvertedSignalToConvertedSignal

	| producer consumer |
	producer := SignalProducerByWrapping new.
	consumer := SignalConsumer new.
	
	"Transform #value: from consumer to signal."
	consumer connect: consumer signal: #value: toSelector: #toggleSwitch.
	"Transform #value: from producer to signal."
	consumer connect: producer signal: #value: toSelector: #value:.
	producer value: #foobar.
	
	self
		assert: #on
		equals: consumer switchState.! !

OrderedCollection subclass: #SignalMethodWrapperRepository
	instanceVariableNames: 'mutex'
	classVariableNames: 'Repository'
	poolDictionaries: ''
	category: 'SI-Wrapper'!

!SignalMethodWrapperRepository class methodsFor: 'instance' stamp: 'mt 6/10/2010 22:41:31.851'!
critical: aBlock

	^ self instance critical: aBlock! !

!SignalMethodWrapperRepository class methodsFor: 'instance' stamp: 'mt 6/10/2010 22:18:23.411'!
instance

	^ Repository ifNil: [Repository := self new] ! !

!SignalMethodWrapperRepository class methodsFor: 'instance' stamp: 'mt 6/14/2010 21:47:55.696'!
reset

	Repository := nil.! !

!SignalMethodWrapperRepository methodsFor: 'critical' stamp: 'mt 6/10/2010 22:30:06.176'!
critical: aBlock

	mutex ifNil: [mutex := Mutex new].
	^ mutex critical: aBlock! !

MwMethodWrapper subclass: #SignalMethodWrapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SI-Wrapper'!

!SignalMethodWrapper commentStamp: 'mt 6/10/2010 22:01:14.237' prior: 0!
I am able to turn any method into a signal without the need of changing the code of that method.!

!SignalMethodWrapper class methodsFor: 'convertig' stamp: 'mt 6/14/2010 21:55:02.808'!
checkAndConvertSignal: aSignalAndClass
	
	(SignalConnectionsRepository checkSignal: aSignalAndClass first inClass: aSignalAndClass second)
		ifFalse:[self convertToSelector: aSignalAndClass first inClass: aSignalAndClass second].! !

!SignalMethodWrapper class methodsFor: 'convertig' stamp: 'mt 6/14/2010 21:54:57.817'!
convertToSelector: aSignal inClass: aClass
	
	SignalMethodWrapperRepository critical: [ | wrapper |
		wrapper := (SignalMethodWrapperRepository instance
			detect: [:w | w wrappedClass = aClass and: [w clientSelector = aSignal]]
			ifNone: [^ self "Is no wrapped signal"]).
		wrapper uninstall.
		SignalMethodWrapperRepository instance remove: wrapper].! !

!SignalMethodWrapper class methodsFor: 'convertig' stamp: 'mt 6/17/2010 15:37:25.871'!
convertToSignal: aSelector inClass: aClass

	| wrapper |
	(aClass whichClassIncludesSelector: aSelector) ifNotNilDo: [ :c |
		c >> aSelector literalsDo: [ :literal |
			(literal == #emit or: [literal == #emitAlways])
				ifTrue: [^ self "Already a signal." ] ] ].
	
	wrapper := self on: aSelector inClass: aClass.
	
	SignalMethodWrapperRepository critical: [
		(SignalMethodWrapperRepository instance anySatisfy: [:w |
			w wrappedClass = aClass and: [w clientSelector = aSelector]]) ifTrue: [^ self "Already a signal via wrapping."].
		SignalMethodWrapperRepository instance add: wrapper].
	
	wrapper install! !

!SignalMethodWrapper methodsFor: 'evaluating' stamp: 'mt 6/10/2010 22:10:34.366'!
valueWithReceiver: anObject arguments: someArguments

	^ [self clientMethod
		valueWithReceiver: anObject
		arguments: someArguments.
	] ensure: [self
		emitSignal: self clientSelector
		arguments: someArguments
		sender: anObject]! !

Object subclass: #SignalProducerByWrapping
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SI-Wrapper-Tests'!

!SignalProducerByWrapping commentStamp: 'mt 6/10/2010 22:53:13.035' prior: 0!
I am a normal value holder.!

!SignalProducerByWrapping class methodsFor: 'wrappers' stamp: 'mt 6/10/2010 22:56:16.402'!
wrapperCount

	^ SignalMethodWrapperRepository instance
		count: [:wrapper | wrapper definingClass == self]! !

!SignalProducerByWrapping methodsFor: 'accessing' stamp: 'mt 6/10/2010 22:53:24.965'!
value

	^ value! !

!SignalProducerByWrapping methodsFor: 'accessing' stamp: 'mt 6/14/2010 18:03:46.994'!
value: aValue
	"Should be used as a signal in tests."

	value := aValue.! !
